def perfect():
    """I'm perfect!"""
    print("I'm perfect!")

def test1():
    """
    """
    print("This is test1.")

def test2():
    # I'm test2.
    print("This is test2.")

# I'm test3.
def test3():
    print("This is test3.")

def test4(): # I'm test4.
    pass
